//Datei: BACKCNT.C
// Hauptprogramm
// Rckwrtstimer mit PIC16F84
//
// Neue Version mit Tastaturabfrage im Interrupt
// und automatischem Rauf-Runterzhlen
//
// hk@holger-klabunde.de
// www.holger-klabunde.de
// 30.04.2001
// Compiler 

// Achtung:	int ist 16 Bit lang
//		char ist 8 Bit lang
//		long ist 32 Bit lang

// Der Takt des PIC betrgt 3.2768MHz
// Ein Maschinenzyklus damit 1.2207us

// Verwendete LCD-Anzeige 2x16 Zeichen
// Bentigt werden nur 1x8 Zeichen !!

#include "prozess.h"      /* Der verwendete Prozessor */

#include "backcnt.h"
#include "protos.h"

 // Watchdog off, Power up Timer on, XT Oscillator, Code Protect off
 __CONFIG(0x3FF1);

//Globale Flags und Rckgabevariablen

unsigned char stelle; 	//Position auf der LCD-Anzeige
unsigned char key;	//Zum Lesen der Tastatur
unsigned char result;	//Zwischenergebnisse
unsigned char tastbuf;  //Tastaturpuffer fr lesen im INT
unsigned char boing;    //Warten auf INT
unsigned char rep;      //Mehrfachauslsen vermeiden
unsigned int countdown; //16Bit Rckwrtszhler
unsigned int tmpcount;  //zum wiederherstellen des alten Wertes
unsigned char gogo;      //Wenn gogo > 0 luft Countdown

/* Hauptprogramm */
void main()
{
 InitPIC(); //Interrupts und so weiter
 LCDInit(); //LCD Anzeige einstellen

 LCDCls();  //Anzeige lschen
 LCDWrite("Holgis");
 LCDPos(2,1);
 LCDWrite("Timer");
 Delay1ms(255);          //Anzeige etwas verlngern
 Delay1ms(255);          //Anzeige etwas verlngern

 LCDCls();  //Anzeige lschen
 LCDPos(2,1);
 LCDWrite("Aus");

 gogo=DISABLE; //Kein Countdown
 
// Hier die gespeicherte Zeit aus dem EEPROM lesen !!
 countdown=0;
 countdown+=Read_EEProm(0); //Lowbyte
 result=Read_EEProm(1);     //Highbyte
 countdown+=(unsigned int)(result<<8);
 tmpcount=countdown; //Den Wert merken fr spter

 stelle=Read_EEProm(2);                //Cursorposition
 if((stelle>6) || (stelle==0)) stelle=1; //Position 1 als Default

 ShowDisplay(); //Aktuellen Wert anzeigen
 
 boing=0; //Flag "TIMERINT war da", sonst wird in der Schleife
 	  //key mehrfach bearbeitet !
 rep=0;   //Merkt sich den letzen Wert von boing
 	  
 while(1) // Anfang Programm Endlosschleife
  {

   if(boing>0) //Taste gedrckt ?
    {
     if(rep!=boing) //War ein neuer INT da ?
      {
       rep=boing;     //nchstes Auslsen erst wieder wenn neuer INT da war

       //Einzelauslsung 
       if(boing==1) key=tastbuf; //Tastaturpuffer lesen, Lowaktiv

       //Ab hier schnelles Tastaturrepeat
       if(boing>50 && (boing&0x1F)==0) key=tastbuf;  
      }
    }
   else rep=0; //Taste losgelassen, Tastenwiederholung lschen
   
   if(key==ONOFF)
    {
     LCDCls();    //Evtl. "Timer an" lschen
     LCDPos(2,1);

     if(lcdport&RELAIS && gogo==DISABLE) //Relais aus erst wenn kein Countdown mehr
      {
       RELAIS_OFF(); //Relais war an dann ausschalten
       LCDWrite("Aus");
      }
     else
      {
       RELAIS_ON();  //Relais war aus dann einschalten
       LCDWrite("An");
      } 

     gogo=DISABLE;  //Countdown abschalten
     countdown=tmpcount; //alten Wert wiederherstellen
     ShowDisplay(); //und anzeigen
    }
   
   if(gogo==DISABLE) //Diese Tasten nur abfragen wenn kein Countdown
    {
     if(key==RIGHT)
      {
       if(stelle<7) stelle++; // Eins nach rechts
       if(stelle==5) stelle=6; // Komma berspringen
       if(stelle==7) stelle=1; // Ans linke Ende des Displays bei berlauf
       ShowDisplay(); 		//Display updaten
      } 

     if(key==UP)
      {
       if(stelle==6 && countdown<65535) countdown++;      //eins hochzhlen
       if(stelle==4 && countdown<=65525) countdown+=10;    //zehn hochzhlen
       if(stelle==3 && countdown<=65435) countdown+=100;   //hundert hochzhlen
       if(stelle==2 && countdown<=64535) countdown+=1000;  //tausend hochzhlen
       if(stelle==1 && countdown<=55535) countdown+=10000; //zehntausend hochzhlen
       ShowDisplay(); 		//Display updaten
       tmpcount=countdown; //Den Wert merken fr spter
      }

     if(key==DOWN)
      {
       if(stelle==6 && countdown>0) countdown--;      //eins runterzhlen
       if(stelle==4 && countdown>=10) countdown-=10;    //zehn runterzhlen
       if(stelle==3 && countdown>=100) countdown-=100;   //hundert runterzhlen
       if(stelle==2 && countdown>=1000) countdown-=1000;  //tausend runterzhlen
       if(stelle==1 && countdown>=10000) countdown-=10000; //zehntausend runterzhlen
       ShowDisplay(); 		//Display updaten
       tmpcount=countdown; //Den Wert merken fr spter
      }

  //Hier STORE Taste auswerten und EEPROM beschreiben
     if(key==STORE)
      {
       LCDPos(2,5);
       LCDWrite("EEP"); //Speichervorgang anzeigen
       Delay1ms(255);
       result=(unsigned char)countdown; //Countdown Lowbyte
       Write_EEProm(0,result);
       result=(unsigned char)(countdown>>8); //Countdown Highbyte
       Write_EEProm(1,result);
       Write_EEProm(2,stelle); //Letzte Cursorposition merken
       LCDPos(2,5);
       LCDWrite("   "); //Store wieder lschen
       LCDPos(1,stelle);
      }

  //Hier START Taste auswerten und Countdown starten
     if(key==START && countdown>0)
      {
       TMR0=0;  //Wer wei wo der gerade stand
       gogo=0;  //Countdown auslsen
       RELAIS_ON();  //Relais war aus dann einschalten
       LCDPos(2,1);
       LCDWrite("Timer an");
      }

     key=0;   //Tastaturinhalt lschen
    } //if gogo==DISABLE
    
   if(gogo<=TICKS) //Countdown luft
    {
     ShowDisplay(); //Auf Display runterzhlen
     Delay1ms(7);  //Sonst wird der Cursor bla
    }

   if(countdown==0&&gogo==SWITCHOFF) //Countdown beendet ?
    {
//     RELAIS_OFF();  //Ja, dann abschalten
     gogo=DISABLE; //countdown erst hier beendet
     countdown=tmpcount; //alten Wert wiederherstellen
     LCDCls();
     LCDPos(2,1);
     LCDWrite("Aus");
     ShowDisplay(); //und anzeigen
    }
    
  } // Ende Programm Endlosschleife
}

//************************************************
// Zeigt die 5 Dezimalstellen auf dem LCD Display 
//************************************************
void ShowDisplay()
{
 LCDPos(1,1);

//Integer zu BCD und anzeigen
 B0=(unsigned char)countdown;      //Lowbyte
 B1=(unsigned char)(countdown>>8); //Highbyte
 BinToBCD16(); //Binr wandeln in BCD Bytes

 //Der Maximalwert ist 6553.5
 LCDWriteByte(R0+0x30);  //Tausender
 LCDWriteByte((R1>>4)+0x30);  //Hunderter
 LCDWriteByte((R1&0x0F)+0x30);  //Zehner

 LCDWriteByte((R2>>4)+0x30);  //Sekunden
 LCDWriteByte(0x2E); //Das Komma
 LCDWriteByte((R2&0x0F)+0x30);  //Zehntelsek.

 LCDWriteByte('s'); //"s" Einheit anzeigen

 LCDPos(1,stelle); // Cursor auf alte Dezimalstelle
}
